#ifndef _COMPUTE
#define _COMPUTE

#ifndef TAC

#include <stdio.h>
#include <stdlib.h>
#include "fourier.h"
#include "glyphs.h"


int Nx=96,Ny=161,Nz=64;
int kx=84,kz=56;
int nwax=kx/2, nwaz=kz/2;



float *get_parse(char *file_name, char *precalc_name, char *glyphs_name)
{
doublecomplex *ut, *vt, *wt;
doublereal *uo, *vo, *wo;

int dim_x = nwax+1;
int dim_y = Ny+1;
int dim_z = 2*nwaz+1;

int my_dim = (dim_x) * (dim_y) * (dim_z);
int max_dim = DIM_X * DIM_Y * DIM_Z;


 ut = (doublecomplex *)malloc(my_dim * sizeof(doublecomplex));
 vt = (doublecomplex *)malloc(my_dim * sizeof(doublecomplex));
 wt = (doublecomplex *)malloc(my_dim * sizeof(doublecomplex));

 uo = (doublereal *)malloc(max_dim * sizeof(doublereal));
 vo = (doublereal *)malloc(max_dim * sizeof(doublereal));
 wo = (doublereal *)malloc(max_dim * sizeof(doublereal));

 if(ut==NULL || vt==NULL || wt==NULL || 
	  uo==NULL || vo==NULL || wo==NULL ) 
 {
	 free(ut);
	 free(vt);
	 free(wt);

	 free(uo);
	 free(vo);
	 free(wo);

  printf("Errore di allocazione della memoria!!\n\n");

  return NULL;
 }


printf("Allocati %.3f MB di dati.\n", (my_dim * sizeof(doublecomplex) + max_dim * sizeof(doublereal))*3.0 / (float)(1024*1024) );

//================

 FILE *fp = fopen( file_name, "rt");

 if(fp == NULL)
 {
	 printf("errore nell'aprire il file di origine! (%s)\n", file_name);
	  free(ut);
	  free(vt);
	  free(wt);

	  free(uo);
	  free(vo);
	  free(wo);
	 return NULL;
 }



printf("\n\n----- Sto caricando in memoria i dati dal file di origine '%s'... ------\n", file_name);


int tot=0;

for(int a=0; a<3; a++)
{
int i,j,k;
int off;
doublecomplex *ptr;
float re, im;

if( a == 0 )  ptr = ut;
if( a == 1 )  ptr = vt;
if( a == 2 )  ptr = wt;

for(k=1; k<=2*nwaz+1; k++)
 for(i=1; i<=nwax+1; i++)
  for(j=0; j<=Ny+1; j++)
	{
   if( !feof(fp) && tot < my_dim )
	 {
		off = j + dim_x * (i + k*dim_y);

    if( fscanf( fp, " (%f,%f)", &re, &im ) == -1 ) break; // nessun dato letto: esci dal ciclo

     ptr[off].r = re;
     ptr[off].i = im;

    tot++;

		if((tot & 0x7FF) == 0) putchar('.');
	 }
	}
	printf("\ncomponenti %c caricate!\n", ('u'+a));
}

	printf(" fatto! %i valori complessi caricati in memoria\n", tot );
	printf(" fattori: ");fattori(tot);


fclose(fp);

//======================

printf("\n------------\nFourier... ");


fourier1_(ut + 7010, vt + 7010, wt + 7010,  uo + 15812, vo + 15812, wo + 15812);
//======================

printf(" OK!!!\n");


	free(ut);
	free(vt);
	free(wt);



printf("salvataggio glyphs ('%s')\n", precalc_name);
// apre i files in cui salvare gli scalari ed i glyphs:
fp = fopen(precalc_name, "wb");

 fwrite(uo, max_dim, sizeof(doublereal), fp);
 fwrite(vo, max_dim, sizeof(doublereal), fp);
 fwrite(wo, max_dim, sizeof(doublereal), fp);

fclose(fp);


//============================================================

glyphs_buffer = (float *)malloc(max_dim * sizeof(float) * 3);

float *scalars = (float *)malloc(max_dim * sizeof(float) );

if( glyphs_buffer == NULL || scalars == NULL )
{
	printf("Errore allocazione della memoria!!\n\n");

	free(uo);
	free(vo);
	free(wo);

	free(glyphs_buffer);
	free(scalars);

	return NULL;
}

printf("Allocati %.3f MB per il glyphs-buffer.\ninizio del fetching-calcolo... ", 
			 (max_dim * sizeof(float) * 3)/(1024.0*1024.0));

float *gb = glyphs_buffer;
float *sb = scalars;
g_avg_val = 0.0;

for(int k=0; k<DIM_Z; k++)
 for(int j=0; j<DIM_Y; j++)
  for(int i=0; i<DIM_X; i++)
	{
		int off = i + DIM_X * (j + k * DIM_Y);
     gb[0] = uo[off];
		 gb[1] = vo[off];
		 gb[2] = wo[off];

     float val = sqrtf( gb[0]*gb[0] + gb[1]*gb[1] + gb[2]*gb[2] );

     if( val < g_min_val ) g_min_val = val;
		  else
		 if( val > g_max_val ) g_max_val = val;

		 g_avg_val += val;


			 *sb = val;

			sb++;

		gb+=3;
	}

	g_avg_val /= (float)(max_dim);

	printf("Calcolo glyphs e scalari terminato:\n - min: %f\n - max: %f\n - avg: %f\n", 
		     g_min_val, g_max_val, g_avg_val);


	free(uo);
	free(vo);
	free(wo);


	printf("inizializzazione Display List glyphs... ");
	 init_glyphs(glyphs_buffer, scalars);
  printf("fatto!\n");

	free(glyphs_buffer);


return scalars;

}


#endif


#endif
