#ifndef _LIGHT_MAT_
#define _LIGHT_MAT_



// #define:

GLint g_lightmat_faces = GL_FRONT_AND_BACK;
#define MAX_LIGHTS     8
#define NUM_MATERIALS 64





// TYPES:
// strutture usate per i colori
typedef struct { GLfloat r, g, b; } t_rgb;
typedef struct { GLfloat r, g, b, a; } t_rgba;

// struttura usata per le coordinate texture
typedef struct { GLfloat u, v; } t_texture_coord;


// definizione di Luce:
typedef struct {
 t_rgba ambient;
 t_rgba diffuse;
 t_rgba specular;

 t_point4d   position; // se si usa GL_LOCAL_VIEWER serve anche w ad 1.0

 t_vector4d direction;

 GLfloat cutoff;

 GLfloat constant;
 GLfloat linear;
 GLfloat quadratic;

 GLenum gl_light; // a quale luce di OpenGL si riferiscono i valori

} t_light;

// definizione di Materiale:
typedef struct {

 t_rgba ambient;
 t_rgba diffuse;
 t_rgba specular;
 t_rgba emission;

 GLfloat shininess;

 GLint gl_textureID;

} t_material;



// VARIABLES:
bool g_lightmat_init_done = false;

t_light    g_lights[MAX_LIGHTS];
t_material g_materials[NUM_MATERIALS];

static t_rgb rainbow[7] = {
	{0,1,1}, // ciano
	{0,0,1}, // blu
	{1,0,1}, // viola
	{1,0,0}, // rosso
	{1,1,0}, // giallo
	{0,1,0}, // verde

	{0,1,1}  // (ciano)
};




// PROTOTYPES::
__inline void color_set3f(t_rgb *c, float r, float g, float b);
__inline void color_set4f(t_rgba *c, float r, float g, float b, float a);
__inline void color_interpolatete3f(t_rgb *dest, t_rgb *c1, t_rgb *c2, float val);
__inline void color_interpolatete4f(t_rgba *dest, t_rgba *c1, t_rgba *c2, float val);

void  color_get_rainbow4f(float s, t_rgba *dest);

void lights_bind_to_opengl();
void lights_enable(t_light *l);
void disable_light(t_light *l);
void lights_bit_enable(short flags8);
void lights_bit_enable_positional(short flags8);
void lights_bit_enable_omni(short flags8);

void init_light_default(t_light *l);

void lights_enableing();

void set_light_val(t_light *l);
__inline void lights_set_position(t_light *l, float x, float y, float z);
__inline void light_set_direction(t_light *l, float dx, float dy, float dz);
__inline void lights_set_at(t_light *l, float x_pos, float y_pos, float z_pos,
						               float target_x, float target_y, float target_z);

__inline void lights_set_diffuse4f(t_light *l, float r, float g, float b, float a);
__inline void lights_set_color4f(t_light *l, float r, float g, float b, float a);


////////////////////////////
void materials_init();
void materials_set(t_material *m);
void materials_get_default_val(t_material *m);





// FUNCTIONS:
__inline void color_set3f(t_rgb *c, float r, float g, float b)
{
 c->r = r;
 c->g = g;
 c->b = b;
}

__inline void color_set4f(t_rgba *c, float r, float g, float b, float a)
{
 c->r = r;
 c->g = g;
 c->b = b;
 c->a = a;
}


__inline void color_interpolatete3f(t_rgb *dest, t_rgb *c1, t_rgb *c2, float val)
{
float k = 1.0 - val;

 dest->r = c1->r * k  +  c2->r * val;
 dest->g = c1->g * k  +  c2->g * val;
 dest->b = c1->b * k  +  c2->b * val;
}

__inline void color_interpolatete4f(t_rgba *dest, t_rgba *c1, t_rgba *c2, float val)
{
float k = 1.0 - val;

 dest->r = c1->r * k  +  c2->r * val;
 dest->g = c1->g * k  +  c2->g * val;
 dest->b = c1->b * k  +  c2->b * val;
 dest->a = c1->a * k  +  c2->a * val;
}




void color_get_rainbow4f(float s, t_rgba *dest)
{
float s1;
float r, g, b;
float r1, g1, b1;
t_rgb *c;

if(!dest) PAUSE;

if(s < 0.0) s = -s;
if(s >= 1.0) s = fmod(s, 1.0);
s*=6.0;

 c = &rainbow[(int)s];

 s  = fmod(s, 1.0);
 s1 = 1.0 - s;

 r1  = c->r;
 g1  = c->g;
 b1  = c->b;

  c++;
 r = c->r;
 g = c->g;
 b = c->b;


 dest->r = r * s  +  r1 * s1;
 dest->g = g * s  +  g1 * s1;
 dest->b = b * s  +  b1 * s1;
                              
}





////////////////////////////
void lights_bind_to_opengl()
{
 g_lights[0].gl_light = GL_LIGHT0;
 g_lights[1].gl_light = GL_LIGHT1;
 g_lights[2].gl_light = GL_LIGHT2;
 g_lights[3].gl_light = GL_LIGHT3;
 g_lights[4].gl_light = GL_LIGHT4;
 g_lights[5].gl_light = GL_LIGHT5;
 g_lights[6].gl_light = GL_LIGHT6;
 g_lights[7].gl_light = GL_LIGHT7;
}

void lights_enable(t_light *l)
{
  glEnable( l->gl_light );
}
void disable_light(t_light *l)
{
  glDisable( l->gl_light );
}


void lights_bit_enable(short flags8)
{
int i;
  for(i=0; i<8; i++)
  {
   if(flags8 & 1) 
	   lights_enable( &g_lights[i] );
        else
		  disable_light( &g_lights[i] );

   flags8>>=1;
  }
}

void lights_bit_enable_positional(short flags8)
{
int i;
  for(i=0; i<8; i++)
  {
   if(flags8 & 1) g_lights[i].position.w = 1.0;
        else
		  g_lights[i].position.w = 0.0;

   glLightfv(g_lights[i].gl_light, GL_POSITION, (const float*)&g_lights[i].position.x);

   flags8>>=1;
  }
}


void lights_bit_enable_omni(short flags8)
{
int i;
  for(i=0; i<8; i++)
  {
   if(flags8 & 1) g_lights[i].cutoff = 180.0;
        else
		  g_lights[i].cutoff = 90.0;

   glLightfv(g_lights[i].gl_light, GL_POSITION, (const float*)&g_lights[i].position.x);

   flags8>>=1;
  }
}


void init_light_default(t_light *l)
{
 color_set4f(&l->ambient, .2, .2, .2, 1);
 color_set4f(&l->diffuse,  1, 1, 1,  1);
 color_set4f(&l->specular, 1, 1, 1,  1);

 l->position.x = 0.0;
 l->position.y = 0.0;
 l->position.z = 2.0;
 l->position.w = 1.0; // va messo a 1.0 se si vuole che sia posizionale con GL_LOCAL_VIEW

 l->direction.x =  0.0;
 l->direction.y =  0.0;
 l->direction.z = -1.0;

 l->cutoff = 180.0;
 //l->cutoff = 60.0;

 l->constant  = 1.0;
 l->linear    = .0125;//(.0125) se != 0.0 aumenta il numero di calcoli per l'illuminazione
 l->quadratic = .125;//(.5) se != 0.0 aumenta il numero di calcoli per l'illuminazione
}

void lighting_init()
{
 int i;
 
if(g_lightmat_init_done) return;

 lights_bind_to_opengl();

 for(i=0; i<MAX_LIGHTS; i++) 
 {
	init_light_default( &g_lights[i] );
	set_light_val( &g_lights[i] );
 }
 //lights_enable( &g_lights[0] );

 g_lightmat_init_done = true;

}


void set_light_val(t_light *l)
{
GLenum gl_light = l->gl_light;

	glLightfv(gl_light, GL_AMBIENT,  (const float*)&l->ambient);
	glLightfv(gl_light, GL_DIFFUSE,  (const float*)&l->diffuse);
    glLightfv(gl_light, GL_SPECULAR,  (const float*)&l->specular);

	glLightfv(gl_light, GL_POSITION,       (const float*)&l->position);
    glLightfv(gl_light, GL_SPOT_DIRECTION, (const float*)&l->direction);

    glLightf(gl_light, GL_SPOT_CUTOFF,     l->cutoff);	

    glLightf(gl_light, GL_CONSTANT_ATTENUATION,  l->constant);
    glLightf(gl_light, GL_LINEAR_ATTENUATION,    l->linear);
    glLightf(gl_light, GL_QUADRATIC_ATTENUATION, l->quadratic);
}




__inline void lights_set_position(t_light *l, float x, float y, float z)
{
 l->position.x = x;
 l->position.y = y;
 l->position.z = z;

 glLightfv(l->gl_light, GL_POSITION, (const float*)&l->position);
}

__inline void light_set_direction(t_light *l, float dx, float dy, float dz)
{
 l->direction.x = dx;
 l->direction.y = dy;
 l->direction.z = dz;
 // per velocizzare le cose (omettendo GL_AUTO_NORMAL o GL_NORMALIZE ?)
 vector3d_norm( (t_vector3d *)&l->direction );
 glLightfv(l->gl_light, GL_SPOT_DIRECTION, (const float*)&l->direction);
}


__inline void lights_set_at(t_light *l, float x_pos, float y_pos, float z_pos,
						               float target_x, float target_y, float target_z)
{

 l->position.x  =  x_pos;
 l->position.y  =  y_pos;
 l->position.z  =  z_pos;
 glLightfv(l->gl_light, GL_POSITION, (const float*)&l->position);

 l->direction.x  =  (target_x  -  x_pos);
 l->direction.y  =  (target_y  -  y_pos);
 l->direction.z  =  (target_z  -  z_pos);
 // per velocizzare le cose (omettendo GL_AUTO_NORMAL o GL_NORMALIZE ?)
 vector3d_norm( (t_vector3d *)&l->direction );
 glLightfv(l->gl_light, GL_SPOT_DIRECTION, (const float*)&l->direction);

}

__inline void lights_set_diffuse4f(t_light *l, float r, float g, float b, float a)
{
 color_set4f(&l->diffuse, r, g, b, a);
 glLightfv(l->gl_light,  GL_DIFFUSE,  (const float*)&l->diffuse);
}


__inline void lights_set_color4f(t_light *l, float r, float g, float b, float a)
{
 color_set4f(&l->diffuse, r, g, b, a);
 glLightfv(l->gl_light,  GL_DIFFUSE,  (const float*)&l->diffuse);
 glLightfv(l->gl_light,  GL_SPECULAR,  (const float*)&l->diffuse);
}







void draw_light(t_light *l)
{// "Disegna" la luce:
glPushMatrix();

 glPushAttrib(GL_ENABLE_BIT);
 glDisable(GL_LIGHTING);
 glDisable(GL_TEXTURE_2D);

//glLoadIdentity();

// 1) disegna la sorgente di luce
 glTranslatef(l->position.x, l->position.y, l->position.z);
 glColor4fv(&l->diffuse.r);
  auxSolidIcosahedron(.08);
  
// 2) ne evidenzia la direzione
 glTranslatef(l->direction.x*.25, 
	          l->direction.y*.25, 
			  l->direction.z*.25 );

glColor4f(.8,.1,.1,.25);
  auxSolidIcosahedron(.04);


 glPopAttrib();

glPopMatrix();
}








//ROUTINES per i materiali:
void materials_init()
{
int i;
  for(i=0; i<NUM_MATERIALS; i++)
  { 
   materials_get_default_val( &g_materials[i] );
  }

 // materials_set( &g_materials[0] );
}


void materials_set(t_material *m)
{
    glMaterialfv(g_lightmat_faces, GL_AMBIENT,   (const float*)&m->ambient);
    glMaterialfv(g_lightmat_faces, GL_DIFFUSE,   (const float*)&m->diffuse);
    glMaterialfv(g_lightmat_faces, GL_SPECULAR,  (const float*)&m->specular);
    glMaterialfv(g_lightmat_faces, GL_EMISSION,  (const float*)&m->emission);

    glMaterialf(g_lightmat_faces, GL_SHININESS,  m->shininess);
}



void materials_get_default_val(t_material *m)
{
 color_set4f(&m->ambient,  1,1,1, 1);
 color_set4f(&m->diffuse,  1,1,1, 1);
 color_set4f(&m->emission, 0,0,0, 1);
 color_set4f(&m->specular, 1,1,1, 1);
 m->shininess = 90.0;
}





#endif


