#ifndef _LIB_TEXTURE
#define _LIB_TEXTURE

#include <stdio.h>

#include "colors.h"


#define NUM_MATERIALS 64

#define NUM_PALETTES 32



// definizione di Materiale:
typedef struct {

 t_rgba ambient;
 t_rgba diffuse;
 t_rgba specular;
 t_rgba emission;

 GLfloat shininess;

 GLuint gl_textureID;

} t_material;



GLubyte g_palette[NUM_PALETTES][256*4];
GLuint  g_palette_id[NUM_PALETTES];


GLenum g_lightmat_faces = GL_FRONT_AND_BACK;
t_material g_materials[NUM_MATERIALS];


void material_set(int id);
void material_get_default(int id);


//ROUTINES per i materiali:
void materials_init()
{
int i;
  for(i=0; i<NUM_MATERIALS; i++)
  { 
   material_get_default( i );
  }

 material_set( 0 );
}

void material_set(int id)
{
t_material *m = &g_materials[id];

    glMaterialfv(g_lightmat_faces, GL_AMBIENT,   (const float*)&m->ambient);
    glMaterialfv(g_lightmat_faces, GL_DIFFUSE,   (const float*)&m->diffuse);
    glMaterialfv(g_lightmat_faces, GL_SPECULAR,  (const float*)&m->specular);
    glMaterialfv(g_lightmat_faces, GL_EMISSION,  (const float*)&m->emission);

    glMaterialf(g_lightmat_faces, GL_SHININESS,  m->shininess);
}

void material_get_default(int id)
{
t_material *m = &g_materials[id];

 color_set4f(&m->ambient,  1,1,1, 1);
 color_set4f(&m->diffuse,  1,1,1, 1);
 color_set4f(&m->emission, 0,0,0, 1);
 color_set4f(&m->specular, 1,1,1, 1);
 m->shininess = 90.0;
}










void rgba256_texture1d(GLuint id, unsigned char *txt)
{
	// Create Linear Filtered Texture
	glBindTexture(GL_TEXTURE_1D, id);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage1D(GL_TEXTURE_1D, 0, 4, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, txt);
}




void init_palettes(void)
{
int i;


printf("Palette generation... ");


GLubyte *ip;

ip = g_palette[0];
float a;
float c[4];
for(i=0, a=0.0; i<256; i++, a+=1.0/256.0)
{
	color_get_ramp4f(a, c, rainbow, 6); 
  for(int j=0;j<4;j++) ip[j] = c[j]*255.0;
}

ip = g_palette[1];
for(i=0, a=0.0; i<256; i++, a+=1.0/256.0)
{
  ip[0] = 255.0-sin(a*3.1415*.5)*255.0;
  ip[1] = a*a*a*255.0;
  ip[2] = 255-a*a*255.0;
	ip[3] = 255.0*sin(a*a*3.1415*.5)/4.0;
  ip+=4;
}



glGenTextures( 2, &g_palette_id[0] );   // Create New Texture

rgba256_texture1d( g_palette_id[0], g_palette[0] );
rgba256_texture1d( g_palette_id[1], g_palette[1] );

glEnable(GL_TEXTURE_1D);
	

printf("ok\n");

}



void update_palette(GLuint palette_id, GLubyte *palette_rgba256)
{
/*
void glTexSubImage1D(GLenum target, GLint level, GLint xoffset,
                     GLsizei width, GLenum format,
                     GLenum type, const GLvoid *pixels);
*/

 glBindTexture( GL_TEXTURE_1D, g_palette_id[palette_id] ); 

 glTexSubImage1D( GL_TEXTURE_1D, 0,  0,256,  GL_RGBA,  GL_UNSIGNED_BYTE, palette_rgba256 );
}


void select_palette(GLuint palette_id )
{
 glBindTexture( GL_TEXTURE_1D, g_palette_id[palette_id] ); 
}


void smap(bool on)
{
if(on)
{
 glEnable(GL_TEXTURE_GEN_S);
 glEnable(GL_TEXTURE_GEN_T);
}
 else
 {
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
 }
}

#endif


///////////////////////////////////////////////////////