// vectors3d.h

#ifndef _LIBRERIA_VETTORI
#define _LIBRERIA_VETTORI

typedef struct { GLfloat x, y, z; } t_point3d;
typedef struct { GLfloat x, y, z; } t_vector3d;

typedef struct { GLfloat x, y, z, w; } t_point4d;
typedef struct { GLfloat x, y, z, w; } t_vector4d;

typedef struct { t_point3d o; t_vector3d u, v, w; } t_basis3d;



// PROTOTYPES::

void  vector3d_make(t_point3d *start, t_point3d *end, t_vector3d *ris);
void  vector3d_prod_v_k(t_vector3d *v, float k);
float vector3d_length(t_vector3d *u);
float vector3d_dot(t_vector3d *u, t_vector3d *v);
void  vector3d_norm(t_vector3d *v);
void  vector3d_norm_k(t_vector3d *v, float k);
void  vector3d_prod_v(t_vector3d * u, t_vector3d * v, t_vector3d * normal);

//////// Funzioni per creazione, gestione e uso delle basi tridimensionali:

void basis3d_make(t_basis3d *b, t_vector3d *y, t_vector3d *top);
void basis3d_transform_point(t_point3d *d, t_point3d *s, t_basis3d *b);

void basis3d_normalize(t_basis3d *b);
void basis3d_normalize_k(t_basis3d *b, float k);
void basis3d_normalize_3k(t_basis3d *b, float k1, float k2, float k3 );

void basis3d_make_normalized(t_basis3d *b, t_vector3d *y, t_vector3d *top);
void basis3d_make_normalized_k(t_basis3d *b, t_vector3d *y, t_vector3d *top, float k);
void basis3d_make_normalized_3k(t_basis3d *b, t_vector3d *y, t_vector3d *top, float k1, float k2, float k3 );

void interpolate_vector(t_vector3d *v1, t_vector3d *v2, float k, t_vector3d *ris);




// FUNCTIONS:



////////////////////////////////////////////
// FUNZIONI PER LE OPERAZIONI SUI VETTORI //
////////////////////////////////////////////

void vector3d_make(t_point3d * start, t_point3d * end, t_vector3d * ris)
{
 ris->x = end->x - start->x;
 ris->y = end->y - start->y;
 ris->z = end->z - start->z;
}

void  vector3d_prod_v_k(t_vector3d *v, float k)
{
 v->x*=k;
 v->y*=k;
 v->z*=k;
}

float vector3d_length (t_vector3d * u)
{
 return sqrtf(u->x * u->x + u->y * u->y + u->z * u->z);
}

float vector3d_dot (t_vector3d * u, t_vector3d * v)
{
 return (u->x * v->x + u->y * v->y + u->z * v->z);
}


void vector3d_norm(t_vector3d *v)
{
float len;
 len = vector3d_length( v );
 if(len)
 {
  len = 1.0/len;
  v->x *= len;
  v->y *= len;
  v->z *= len;
 }
}

void vector3d_norm_k(t_vector3d *v, float k)
{
float len;
 len = vector3d_length( v );
 if(len)
 {
  len = k / len;
  v->x *= len;
  v->y *= len;
  v->z *= len;
 }
}


void  vector3d_prod_v (t_vector3d * u, t_vector3d * v, t_vector3d * normal)
{
  normal->x = (u->y * v->z) - (u->z * v->y);
  normal->y = (u->z * v->x) - (u->x * v->z);
  normal->z = (u->x * v->y) - (u->y * v->x);

  //vector3d_norm( normal );
}



//////// Funzioni per makezione, gestione e uso delle basi tridimensionali:


void basis3d_make(t_basis3d *b, t_vector3d *y, t_vector3d *top)
{
 b->v = *y;
 vector3d_prod_v( top, y,  &b->u );
 vector3d_prod_v( &b->u, &b->v, &b->w );
}


void basis3d_transform_point(t_point3d *d, t_point3d *s, t_basis3d *b)
{
  d->x = s->x * b->u.x  +  s->y * b->v.x  +  s->z * b->w.x  +  b->o.x;
  d->y = s->x * b->u.y  +  s->y * b->v.y  +  s->z * b->w.y  +  b->o.y;
  d->z = s->x * b->u.z  +  s->y * b->v.z  +  s->z * b->w.z  +  b->o.z;
}


void basis3d_normalize(t_basis3d *b)
{
 vector3d_norm( &b->u );
 vector3d_norm( &b->v );
 vector3d_norm( &b->w ); 
}

void basis3d_normalize_k(t_basis3d *b, float k)
{
 vector3d_norm_k( &b->u, k );
 vector3d_norm_k( &b->v, k );
 vector3d_norm_k( &b->w, k ); 
}

void basis3d_normalize_3k(t_basis3d *b, float k1, float k2, float k3 )
{
 vector3d_norm_k( &b->u, k1 );
 vector3d_norm_k( &b->v, k2 );
 vector3d_norm_k( &b->w, k3 ); 
}

void basis3d_make_normalized(t_basis3d *b, t_vector3d *y, t_vector3d *top)
{
 b->v = *y;
 vector3d_prod_v( top, y,  &b->u );
 vector3d_prod_v( &b->u, &b->v, &b->w );
 basis3d_normalize( b );
}

void basis3d_make_normalized_k(t_basis3d *b, t_vector3d *y, t_vector3d *top, float k)
{
 b->v = *y;
 vector3d_prod_v( top, y,  &b->u );
 vector3d_prod_v( &b->u, &b->v, &b->w );
 basis3d_normalize_k( b, k );
}

void basis3d_make_normalized_3k(t_basis3d *b, t_vector3d *y, t_vector3d *top, float k1, float k2, float k3 )
{
 b->v = *y;
 vector3d_prod_v( top, y,  &b->u );
 vector3d_prod_v( &b->u, &b->v, &b->w );
 basis3d_normalize_3k( b,  k1, k2, k3 );
}




void interpolate_vector(t_vector3d *v1, t_vector3d *v2, float k, t_vector3d *ris)
{
float k1 = 1.0f - k;
 ris->x = v1->x * k1  +  v2->x * k;
 ris->y = v1->y * k1  +  v2->y * k;
 ris->z = v1->z * k1  +  v2->z * k;

 //vector3d_norm(ris);
}

void interpolate_vector_n(t_vector3d *v1, t_vector3d *v2, float k, t_vector3d *ris)
{
float k1 = 1.0f - k;
 ris->x = v1->x * k1  +  v2->x * k;
 ris->y = v1->y * k1  +  v2->y * k;
 ris->z = v1->z * k1  +  v2->z * k;

  vector3d_norm(ris);
}



#endif

